char junk;  // junk code, included to avoid arduino bug
/*
 * File:           C:\Users\wsaba\Google Drive\Senior Project\uC and DSP Code\EXPERIMENT\experiment.ino
 *
 * Created:        Monday, June 01, 2015 9:43:02 AM
 * Description:    Main, Filter Import
 *
 * This software is distributed in the hope that it will be useful,
 * but is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 *
 * Purpose: This file controls the menu, LCD driver, and filter creation
 *
*/

/*******************--#includes - Libraries--*********************/
// ---------------i2c communication----------------------------//
#include <Wire.h>
// ---------------EEPROM Preset Reading------------------------//
#include <EEPROM.h>
//----------------DSP Interaction Functions--------------------//
#include "SigmaStudioFw.h"
//---------------- #defines for DSP Control Register Parameters--------------------//
#include "main_final_IC_1_REG.h"
//----------------STDIO Equivalent for Arduino-----------------//
//#include <avr/io.h>
//----------------Biquad Calculation, Biquad Class-------------//
#include "Biquad.h"
//----------------DSP Boot Sequence and Register Storage-------// 
#include "main_final_IC_1.h"
//----------------#defines for DSP Parameter Registers (Name Based References)------//
#include "main_final_IC_1_PARAM.h"
//----------------Necessary for using the 7-segment display----//
//#include "LCDDriver.h"
//----------------Rotary Encoder Handling----------------------//
#include "RotaryEncoder.h"
/*******************End ->#includes - Libraries--*********************/


//Set samplerate for filter calculation
#define sampleRate 96000.0
// i2c Write address of the ADAU DSP board
#define I2C_ADDR_ADAU1702W 0x34 
//#defines corresponding to enumerated filter types for menu-display and code readability//
#define  type_peak 0
#define  type_lowshelf 1
#define  type_highshelf 2
#define  type_notch 3
//Return value indicating that the Import File designates subwoofer mode//
#define  SUB_MODE_FLAG_HIGH 2
// Default crossover Fc, in Hz
#define DEFAULT_CROSSOVER_FC 200 
//DSP registers stored in progmem for reduced dynamic memory access, read out via default download -- boot sequence
//typedef used to visually indicate these variables and to match default_download type format//
typedef PROGMEM const unsigned char  ADI_REG_TYPE;

/*******************--Global Variables--*********************/

//8 biquads necessary to implement 4th order crossover, LP = low pass, HP = high pass, 1 = 1st filter, 
// 2 = 2nd filter (order in cascade), R = right channel, L = left channel 
   Biquad *LP1L = new Biquad();
   Biquad *LP2L = new Biquad();
   Biquad *LP1R = new Biquad();
   Biquad *LP2R = new Biquad();
   Biquad *HP1L = new Biquad();
   Biquad *HP2L = new Biquad();
   Biquad *HP1R = new Biquad();
   Biquad *HP2R = new Biquad();
   
//Default Volume, initialized in Setup(), this is the index, points to dBLookup
int volume = 37;
// -40 -> 0 dB lookup table stored in decimal format for volume control and wherever -dB gain is needed as a decimal (dB values stored as linear decimals in registers)
double dBLookup [41] = {0.01,0.011220185,0.012589254,0.014125375,0.015848932,0.017782794,0.019952623,0.022387211,0.025118864,0.028183829,0.031622777,0.035481339,0.039810717,0.044668359,0.050118723,0.056234133,0.063095734,0.070794578,0.079432823,0.089125094,0.1,0.112201845,0.125892541,0.141253754,0.158489319,0.177827941,0.199526231,0.223872114,0.251188643,0.281838293,0.316227766,0.354813389,0.398107171,0.446683592,0.501187234,0.562341325,0.630957344,0.707945784,0.794328235,0.891250938,1};

// Globals that handle menu operations
int is_pressed = 0;
int is_released = 0;
int push_turn = 0;
int turn_dir = 0;
int pos = 0;
// Index responsible for handling menu crossover adjustment, points to fcLookup
int crossoverFc_i = 34; // 

// Display buffer
char buf[20];
//Initialize Analog inputs 2 and 3 as encoder L and R
RotaryEncoder encoder(A2, A3);
volatile int count = 0;
//Declare menu variable
int menu[5] = {0,0,0,0,0};

/*******************--END ->Global Variables--*********************/

/*******************--Function Declarations--*********************/

void setup() {
   Biquad *filter1  = new Biquad();
   Biquad *filter2  = new Biquad();
   Biquad *filter3  = new Biquad();
   Biquad *filter4  = new Biquad();
   Biquad *filter5  = new Biquad();
   Biquad *filter6  = new Biquad();  
   Biquad *filter7  = new Biquad();
   Biquad *filter8  = new Biquad();
   Biquad *filter9  = new Biquad();
   Biquad *filter10 = new Biquad();
   
   int subMode = 0;// temporary, testing submode

  //Wait for DSP to initialize clock
  delay(350);
//Write "firmware" version of code to DSP
Serial.begin(9600);
  default_download_IC_1();
  Serial.end();
//Wait for DSP to load memory
  delay(100);

//Diagnostic display delay
  delay(100);
//Pullup on rotary press enable
  pinMode(2, INPUT_PULLUP);
  sei();
  PCICR |= (1 << PCIE1);   // This enables Pin Change Interrupt 1 that covers the Analog input pins or Port C.
  PCICR |= (1 << PCIE2);   //This enables Pin Change Interrupt 2 that covers Port D.
  PCIFR |= (1 << PCIF2);   //Enables a flag in response to logic change to trigger ISR2. Flag cleared after completion of ISR. 
  PCMSK1 |= (1 << PCINT10) | (1 << PCINT11);  // This enables the interrupt for pin 2 and 3 of Port C.
  PCMSK2 |= (1 << PCINT18);                   // This enables the interrupt for pin 2 of Port D.

  
/*******************--Load Default DSP filters (different from firmware version)--*********************/

  setVolume(dBLookup[volume], I2C_ADDR_ADAU1702W);
//Initialize crossover to DEFAULT_CROSSOVER_FC Hz LR 24 dB/octave//

    LP1L ->setBiquad(bq_type_lowpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    LP2L ->setBiquad(bq_type_lowpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    LP1R ->setBiquad(bq_type_lowpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    LP2R ->setBiquad(bq_type_lowpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    HP1L ->setBiquad(bq_type_highpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    HP2L ->setBiquad(bq_type_highpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    HP1R ->setBiquad(bq_type_highpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);
    HP2R ->setBiquad(bq_type_highpassLR, DEFAULT_CROSSOVER_FC / sampleRate, 1, 0);  
   
    setCrossover(DEFAULT_CROSSOVER_FC/ sampleRate, LP1L, LP1R,LP2L,LP2R,HP1L,HP1R,HP2L,HP2R);
    // load text file function// 

//  Import filters from Room Equalizer Wizard export format text pattern, Text is currently stored in EEPROM for testing purposes starting at address 0
    subMode = importFilter(filter1,filter2,filter3,filter4,filter5,filter6,filter7,filter8,filter9,filter10); // 
 // note that filter1 starts at "stage0" so filter names are stage+1
    safeLoadFilter(filter1, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE0_B0_ADDR);
    safeLoadFilter(filter2, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE1_B0_ADDR); 
    safeLoadFilter(filter3, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE2_B0_ADDR);
    safeLoadFilter(filter4, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE3_B0_ADDR);
    safeLoadFilter(filter5, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE4_B0_ADDR);
    safeLoadFilter(filter6, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE5_B0_ADDR);
    safeLoadFilter(filter7, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE6_B0_ADDR);
    safeLoadFilter(filter8, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE7_B0_ADDR);
    safeLoadFilter(filter9, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE8_B0_ADDR);
    safeLoadFilter(filter10, MOD_2NDORDEREQFILTERBANK_ALG0_STAGE9_B0_ADDR);
//Convert Left and Right (low) outputs to a singular left low mono subwoofer output by ...
//   Toggling 2 mutes and adjusting mux gain to compensate for the 2x gain of the summed signals
if(subMode == SUB_MODE_FLAG_HIGH){
      subModeEnable(MOD_SUBMODETOGGLE_MUTENOSLEWALG2MUTE_ADDR,MOD_RIGHTMUTEIFSUB_MUTENOSLEWALG3MUTE_ADDR,MOD_SUBMODEGAINADJ_SINGLECTRLMIXERNEW19401_ADDR);
    }

 /***************-- Example Function Calls for Updating DSP Parameters--******************************/
/*
  //Example function calls
  updateHP(hpLookup_loc, 0, DEVICE_ADDR_IC_1, MOD_HIPASSPROTECT_ALG0_STAGE0_B0_ADDR);
  setLimiter(dBLookup[limiterIndex-15], DEVICE_ADDR_IC_1, MOD_LIMITERRIGHT_LOW_ALG0_THRESHOLD_ADDR,MOD_LIMITERLEFT_LOW_ALG0_THRESHOLD_ADDR);
  setVolume(dBLookup[0], DEVICE_ADDR_IC_1, MOD_SWVOL1_ALG0_TARGET_ADDR,MOD_SWVOL1_ALG0_STEP_ADDR);
  filter1 ->setBiquad(bq_type_peak, (double)pgm_read_word_near(fcLookup+filter1i.fc-65) / sampleRate, qLookup[filter1i.q], gainLookup[filter1i.gain+80]);
*/   
  
    
    
}

// ISRs for handling push, and rotate for rotary encoder
ISR(PCINT1_vect) {
  noInterrupts();
  encoder.tick(); // just call tick() to check the state.
  interrupts();
}

//Debouncing issue, ISR keeps getting called on logic change
ISR(PCINT2_vect) {               
   noInterrupts();
   delay(50);
   if(digitalRead(2) == 0)   //active low button aka press down
   {
     is_pressed = 1;  
   }
   else                      // button release 
   {
     is_released = 1;
   }
   interrupts();
}

void loop() {

  
  int newPos = encoder.getPosition();
  if (newPos != pos) {
    //Serial.print(newPos);
    //Serial.println();
    turn_dir = newPos - pos;
    pos = newPos;
    }
  
  //Variable updates depending on whether turn right/left or button 
  if(turn_dir || is_pressed || is_released) //if any menu change interrupt occurs
  {
//    menu_update();  //change menu
    turn_dir = 0;   //reset position vector
  }  
}

